package org.netkernel.xml.accessor.schema;

import java.io.ByteArrayInputStream;
import java.io.IOException;

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;
import org.netkernel.xml.util.NKFEntityResolver;
import org.xml.sax.ErrorHandler;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.helpers.DefaultHandler;

public class ValidateDTDAccessor extends StandardAccessorImpl
{
	public ValidateDTDAccessor()
	{	this.declareThreadSafe();		
	}

	@Override
	public void onSource(INKFRequestContext context) throws Exception
	{	StringBuilder b=new StringBuilder();
		b.append(context.source("arg:operator", String.class));
		b.append("\n");
		b.append(context.source("arg:operand", String.class).replaceAll("<\\?xml.*?>", ""));
		
		SAXParserFactory spf=SAXParserFactory.newInstance();
		
		spf.setFeature("http://xml.org/sax/features/validation",true);
		spf.setFeature("http://xml.org/sax/features/external-parameter-entities",true);
		spf.setFeature("http://apache.org/xml/features/nonvalidating/load-dtd-grammar",true);
		spf.setFeature("http://apache.org/xml/features/nonvalidating/load-external-dtd",true);
		spf.setNamespaceAware(true);
        spf.setValidating(true);
        
        SAXParser sp=spf.newSAXParser();
        
        LocalErrorHandler handler=new LocalErrorHandler(context);
        sp.parse(new ByteArrayInputStream(b.toString().getBytes()), handler);
        context.createResponseFrom(handler.errors==0);
	}
	
	private class LocalErrorHandler extends DefaultHandler
	{	
		public int errors=0;
		private INKFRequestContext mContext;
		private NKFEntityResolver mResolver;
		
		public LocalErrorHandler(INKFRequestContext context)
		{	mContext=context;			
			mResolver=new NKFEntityResolver(context);
		}

		@Override
		public void error(SAXParseException e) throws SAXException
		{	errors++;
			mContext.logRaw(INKFRequestContext.LEVEL_WARNING, e.toString());
		}

		@Override
		public void fatalError(SAXParseException e) throws SAXException
		{	errors++;
			mContext.logRaw(INKFRequestContext.LEVEL_WARNING, e.toString());
		}

		@Override
		public void warning(SAXParseException e) throws SAXException
		{	mContext.logRaw(INKFRequestContext.LEVEL_WARNING, e.toString());
		}

		@Override
		public InputSource resolveEntity(String publicId, String systemId)
				throws IOException, SAXException
		{	return mResolver.resolveEntity(publicId, systemId);
		}
	}
}
