/******************************************************************************
  (c) Copyright 2002,2003, 1060 Research Ltd                                   

  This Software is licensed to You, the licensee, for use under the terms of   
  the 1060 Public License v1.0. Please read and agree to the 1060 Public       
  License v1.0 [www.1060research.com/license] before using or redistributing   
  this software.                                                               

  In summary the 1060 Public license has the following conditions.             
  A. You may use the Software free of charge provided you agree to the terms   
  laid out in the 1060 Public License v1.0                                     
  B. You are only permitted to use the Software with components or applications
  that provide you with OSI Certified Open Source Code [www.opensource.org], or
  for which licensing has been approved by 1060 Research Limited.              
  You may write your own software for execution by this Software provided any  
  distribution of your software with this Software complies with terms set out 
  in section 2 of the 1060 Public License v1.0                                 
  C. You may redistribute the Software provided you comply with the terms of   
  the 1060 Public License v1.0 and that no warranty is implied or given.       
  D. If you find you are unable to comply with this license you may seek to    
  obtain an alternative license from 1060 Research Limited by contacting       
  license@1060research.com or by visiting www.1060research.com                 

  NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC  
  LICENSE V1.0 FOR DETAILS                                                     

  THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ     
  THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license      

  File:          $RCSfile: NKFURIResolver.java,v $
  Version:       $Name:  $ $Revision: 1.1 $
  Last Modified: $Date: 2008/04/21 09:38:02 $
 *****************************************************************************/
package org.netkernel.xml.util;

import org.netkernel.layer0.nkf.*;
import org.netkernel.layer0.representation.IBinaryStreamRepresentation;
import org.netkernel.layer0.representation.IReadableBinaryStreamRepresentation;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;

import javax.xml.transform.sax.SAXSource;
import javax.xml.transform.stream.*;
import javax.xml.transform.dom.*;
import javax.xml.transform.*;

import java.io.*;
import java.net.*;
/**
 * Implementation of a javax.xml.transform.URIResolver using NKF context
 * @author  tab
 */
public class NKFURIResolver implements javax.xml.transform.URIResolver
{
	private INKFRequestContext mContext;
	
	/** Creates a new instance of NKFURIResolver */
	public NKFURIResolver(INKFRequestContext aContext)
	{	mContext=aContext;
	}
	
	/** ensure link to context is removed as some XML technologies
	 don't like to let go
	 */
	public void cleanup()
	{	mContext=null;
	}
	
	public Source resolve(String aHref, String aBase) throws TransformerException
	{	try
		{	if(aHref.equals(""))
			{	aHref=aBase;
			}
			URI hrefUri=URI.create(aHref);
			URI ref=null;
			if(hrefUri.isAbsolute())
			{	ref=hrefUri;
			}
			else
			{	if(aBase!=null)
				{	URI baseUri=URI.create(aBase);
					ref=baseUri.resolve(hrefUri);
				}
				else
				{	ref=URI.create(mContext.getCWU());				
				}
			}
			String  resource=ref.toString();
			Object resourceRep = mContext.source(resource);
			Source result;
			if(resourceRep instanceof Document)
			{	Document d = (Document)resourceRep;
				result=new DOMSource(d);
			}
			else if(resourceRep instanceof IReadableBinaryStreamRepresentation)
			{	IReadableBinaryStreamRepresentation rbs = (IReadableBinaryStreamRepresentation)resourceRep;
				result=getSAXSource(rbs.getInputStream());
			}
			else if(resourceRep instanceof IBinaryStreamRepresentation)
			{	ByteArrayOutputStream baos=new ByteArrayOutputStream(2048);
				IBinaryStreamRepresentation bs=(IBinaryStreamRepresentation)resourceRep;
				bs.write(baos);
				result=getSAXSource(new ByteArrayInputStream(baos.toByteArray()));
			}
			else 
			{	//Worst case, try to tranrept the resource to a readable binary stream
				IReadableBinaryStreamRepresentation rbs = (IReadableBinaryStreamRepresentation)mContext.source(aHref, IReadableBinaryStreamRepresentation.class);
				result=getSAXSource(rbs.getInputStream());
			}
			if(result!=null)
			{	result.setSystemId(resource);
			}
			return result;
		} catch (NKFException e)
		{	String msg=mContext.formatMessage("MSG_XML_NO_RESOLVE", new Object[]{aHref});
			TransformerException e2 = new TransformerException(msg);
			e2.initCause(e);
			throw e2;
		}
		catch (IOException e)
		{	String msg=mContext.formatMessage("MSG_XML_NO_RESOLVE", new Object[]{aHref});
			TransformerException e2 = new TransformerException(msg);
			e2.initCause(e);
			throw e2;
		}
	}
	
	private SAXSource getSAXSource(InputStream aStream) throws TransformerException
	{	XMLReader reader=null;
		try
		{	reader=XMLReaderFactory.createXMLReader();
		}
		catch(SAXException e)
		{	throw new TransformerException(e.getMessage());
		}
		reader.setEntityResolver(new NKFEntityResolver(mContext));
		InputSource src=new InputSource(aStream);
		return new SAXSource(reader, src);
	}
}